;; -*- lexical-binding: t; -*-

(defun th/display-buffer-in-my-pdf-frame (buffer alist)
  (let ((pred (lambda (f) (frame-parameter f 'th/pdf-frame))))
    (if-let ((frame (car (seq-filter pred (frame-list)))))
        (display-buffer-use-some-frame
         buffer `((frame-predicate . ,pred)))
      (display-buffer-pop-up-frame
       buffer (cons
               '(pop-up-frame-parameters (th/pdf-frame . t))
               alist)))))

(defun th/display-buffer-but-not-in-my-pdf-frame (buffer alist)
  (let ((pred (lambda (f)
                (and (frame-visible-p f)
                     (null (frame-parameter f 'th/pdf-frame))))))
    (if-let ((frame (car (seq-filter pred (frame-list)))))
        (display-buffer-use-some-frame
         buffer `((frame-predicate . ,pred)))
      (display-buffer-pop-up-frame buffer alist))))

;; PDFs in a dedicated pdf frame.
(add-to-list 'display-buffer-alist
             `((derived-mode . pdf-view-mode)
               th/display-buffer-in-my-pdf-frame))

;; Non-PDF buffers not in the pdf-frame
(add-to-list 'display-buffer-alist
             `((not (derived-mode . pdf-view-mode))
               th/display-buffer-but-not-in-my-pdf-frame))

;; Make backward search show the tex buffer in another non-PDF frame if it's
;; already shown there.
(setq pdf-sync-backward-display-action #'th/display-buffer-but-not-in-my-pdf-frame)

;; The below hopefully handles the case where emacs is called with the pdf file
;; from the command line/file manager, so that the inital frame must already
;; get the th/pdf-frame parameter.
(defun th/pdf-frame-ensure ()
  (when (and (derived-mode-p 'pdf-view-mode)
             (null (seq-filter
                    (lambda (f) (frame-parameter f 'th/pdf-frame))
                    (frame-list))))
    (set-frame-parameter
     (window-frame (get-buffer-window (current-buffer) 'visible))
     'th/pdf-frame t)))

(add-hook 'window-setup-hook #'th/pdf-frame-ensure)
